/*
MyPluvio rain gauge with tipping bucket version 1.
Alex Pikkert June 2025.
6kHz pulses are send by the 433MHz remote rain gauge and filtered out with an array.
the feed (A) is sent to Adafruit IO via WiFi with an ESP01.
feed factor calculation:
1 feed is the amount of rain volume in mm3 per tipping divided by the surface of the gauge in mm2.
rain volume per tipping is 2.5 cm3 (2500 mm3).
gauge surface: the diameter is 110 mm. the radius is 55 mm. 
the surface is pi x square radius: 3.14 x 55 x 55 = 9503 mm2
tippingFeed is volume/surface: 2500/9503 = 0.263.
*/

#include <ESP8266WiFi.h>
#include "AdafruitIO_WiFi.h"
int i = 0;
const int ArraySize = 10;            // aantal frequentiemetingen
const int pulsePin = 2;              // De pin waar de pulsen binnenkomen
const int rainPin = 0;               // De pin waar de echte gecheckte puls te zien is
unsigned long pulseHigh;             // Hoogse tijd (microseconden)
unsigned long pulseLow;              // Laagse tijd (microseconden)
unsigned long pulseTotal;            // Totale tijd van de puls
unsigned long frequency;             // Gemeten frequentie in Hertz
unsigned long FreqArray[ArraySize];  // Opslag van de metingen

// WiFi instellingen
#define WIFI_SSID       "your wireless network name (SSID)"
#define WIFI_PASS       "your wireless network password"

// Adafruit IO instellingen
#define IO_USERNAME     "your Adafruit IO account user name"
#define IO_KEY          "your Adafruit IO key"

// Maak een Adafruit IO WiFi client aan
AdafruitIO_WiFi io(IO_USERNAME, IO_KEY, WIFI_SSID, WIFI_PASS);

float A = 0;
unsigned long huidigeTijd = 0;
unsigned long vorigeTijd = 0;
unsigned long interval = 3600000; // 1 uur tussen elke feed update naar het internet
unsigned long vorigeTijd1 = 0;

// Bepaal de periode waarna een reset plaatsvindt
//unsigned long interval1 = 2592000000; // 30 dagen meten waarna meting op nul gezet wordt
//unsigned long interval1 = 604800000 ; // 1 week meten waarna meting op nul gezet wordt
unsigned long interval1   = 86400000;     // 24 uur meten waarna meting op nul gezet wordt
//unsigned long interval1 = 43200000;   // 12 uur meten waarna meting op nul gezet wordt

// feed
AdafruitIO_Feed *floatA = io.feed("floatA");

void setup() {
  Serial.begin(115200);
  delay(1000);
  pinMode(pulsePin, INPUT_PULLUP);          // Stel de puls pin in als input
  pinMode (rainPin, OUTPUT);

  // Start verbinding met Adafruit IO
  io.connect();
  while (io.status() < AIO_CONNECTED) {
    Serial.print(".");
    delay(500);
  }
  Serial.println("Verbonden!");

  for (i = 0; i < 3; i++) {
    digitalWrite (rainPin, HIGH);
    delay(200);
    digitalWrite (rainPin, LOW);
    delay(200);
  }

}

void loop() {
  io.run();
  CheckPulse();
  huidigeTijd = millis();
  if (huidigeTijd - vorigeTijd > interval) {
    vorigeTijd = huidigeTijd;

    // Verstuur de waarde naar Adafruit IO
    Serial.print("Verstuur waarde A: ");
    Serial.println(A);
    floatA->save(A);
  }

  huidigeTijd = millis();
  if (huidigeTijd - vorigeTijd1 > interval1) {
    vorigeTijd1 = huidigeTijd;
    A = 0;
  }
}

//*******************************************************************************

void CheckPulse() {
  for (i = 0; i < ArraySize; i++) {
    pulseHigh = pulseIn(pulsePin, HIGH);   // Meet de tijd voor de hoge puls
    pulseLow = pulseIn(pulsePin, LOW);     // Meet de tijd voor de lage puls
    pulseTotal = pulseHigh + pulseLow;     // Totale periode van de puls
    if (pulseTotal > 0) {
      frequency = 1000000 / pulseTotal;    // Bereken de frequentie in Hertz
    }
    if (frequency > 5000 && frequency < 7000 ) {
      FreqArray[i] = frequency;
    }
    else FreqArray[i] = 0;
  }
  for (i = 0; i < ArraySize; i++) {
    frequency = frequency + FreqArray[i];
  }
  frequency = frequency / ArraySize;
  if (frequency > 5000 && frequency < 7000 ) {
    digitalWrite (rainPin, HIGH);
    Serial.println(frequency);
    delay(2000);
    digitalWrite (rainPin, LOW);
    A = A + 0.263;
  }
}
